module PPADDR(CLK, WE, RE, DATA, ADDR, AW, AR);
input CLK;
input WE;
input RE;
input DATA;
output [7:0] ADDR;
output AW;
output AR;

reg [7:0] ADDR;
reg AW, AR;
reg [1:0] state;

// Clock based logic
always @(posedge CLK) begin
    // Reset the addres when both enables are low
    if (~WE & ~RE) begin
        ADDR <= 'h0;
        AW <= 'h0;
        AR <= 'h0;
        state <= 0;
    end
    // Ready to receive a byte
    else if (WE) begin
        // State machine
        case (state)
          // @(posedge DATA) -- Store and ACK the byte
          0: if (DATA) begin AW <= 'h1; state <= 3; end
          
          // Un-ACK and Increment the address
          1: begin AW <= 'h0; ADDR <= ADDR + 'h1; state <= 0; end
          
          // @(negedge DATA) -- The byte has been written
          2: if (~DATA) state <= 1;
          
          // One cycle to store the data
          3: state <= 2;
        endcase
    end
    // Ready to send a byte
    else if (RE) begin
        // State machine
        case (state)
          // @(posedge DATA) -- Send the byte
          0: if (DATA) begin AR <= 'h1; state <= 2; end
          
          // Un-ACK and Increment the address
          1: begin AR <= 'h0; ADDR <= ADDR + 'h1; state <= 0; end
          
          // @(negedge DATA) -- The byte has been read
          2: if (~DATA) state <= 1;
        endcase
    end
end
endmodule
